﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "到账冲账")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class PaymentOffset : RestfulExecution<Payment>
    {
        public Guid Kvid { get; set; }

        public string Remark { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty).ThrowIfTrue("请传入要冲账的到账Kvid!");
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payment = conn.SingleById<Payment>(Kvid);
            if (payment == null) throw new Exception("收付款项不存在");
            if (payment.Type != PaymentType.Bank && payment.Type != PaymentType.Cash && payment.Type != PaymentType.Pos && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat) throw new Exception("非可冲销收款类型!");
            if (payment.BizKvid != Guid.Empty) throw new Exception("收付款已关联业务！请先解除关联！");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("收付款已冲账");
            if (payment.AmountInvoice > 0 || payment.AmountSplited > 0 || payment.AmountUsed > 0) throw new Exception($"不可冲账,此到账已开票:{payment.AmountInvoice}元,已认领:{payment.AmountSplited}元,已使用:{payment.AmountUsed}元");
            //查询当前项的拆分子项List
            if (conn.Exists<Payment>(o => o.ParentKvid == Kvid && o.OffsetKvid == Guid.Empty)) throw new Exception("不可冲账,此到账存在未冲账子项!");
            if (conn.Exists<InvoiceApply>(o => o.BizKvid == payment.Kvid && o.OffsetKvid == Guid.Empty)) throw new Exception("申请开票中,不可冲账!");
            payment.PayeeAccountKvid.ThrowIfEmpty("收款账户(PayeeAccountKvid)为空!");
            payment.PayerAccountKvid.ThrowIfEmpty("付款账户(PayerAccountKvid)为空!");

            var accountPayee = conn.SingleById<Account>(payment.PayeeAccountKvid);
            var accountPayer = conn.SingleById<Account>(payment.PayerAccountKvid);

            if (accountPayee == null) accountPayee = payment.Currency.GetBalanceAccount();
            if (accountPayer == null) accountPayer = payment.Currency.GetBalanceAccount();

            var offset = payment.Offset($"冲账:{Remark}");

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            using (var trans = conn.OpenTransaction())
            {
                conn.UpdateOnly(payment);
                rtns.Results.Add(payment);
                conn.Insert(offset);

                if (accountPayee != null)
                {
                    #region 新增流水
                    var newAccountDetail = new AccountDetail();
                    newAccountDetail.AccountKvid = accountPayee.Kvid;
                    newAccountDetail.BizId = payment.SerialNumber;
                    newAccountDetail.BizKvid = payment.Kvid;
                    newAccountDetail.BizType = typeof(Payment).FullName;

                    newAccountDetail.PayerKvid = payment.PayeeKvid;
                    newAccountDetail.PayerName = payment.PayeeName;
                    newAccountDetail.PayerAccountKvid = accountPayee.Kvid;
                    newAccountDetail.PayerAccountName = accountPayee.Name;
                    newAccountDetail.PayerAccountSerialNumber = accountPayee.SerialNumber;

                    newAccountDetail.PayeeKvid = payment.PayerKvid;
                    newAccountDetail.PayeeName = payment.PayerName;
                    newAccountDetail.PayeeAccountKvid = accountPayer.Kvid;
                    newAccountDetail.PayeeAccountName = accountPayer.Name;
                    newAccountDetail.PayeeAccountSerialNumber = accountPayer.SerialNumber;

                    newAccountDetail.AmountPayment = payment.Amount;
                    newAccountDetail.Summary = "冲账:" + Remark;
                    #endregion
                    newAccountDetail.Insert(conn);
                }
                if (accountPayer != null)
                {
                    #region 新增流水
                    var accountDetailBalance = new AccountDetail();
                    accountDetailBalance.AccountKvid = accountPayer.Kvid;
                    accountDetailBalance.BizId = payment.SerialNumber;
                    accountDetailBalance.BizKvid = payment.Kvid;
                    accountDetailBalance.BizType = typeof(Payment).FullName;

                    accountDetailBalance.PayerKvid = payment.PayeeKvid;
                    accountDetailBalance.PayerName = payment.PayeeName;
                    accountDetailBalance.PayerAccountKvid = accountPayee.Kvid;
                    accountDetailBalance.PayerAccountName = accountPayee.Name;
                    accountDetailBalance.PayerAccountSerialNumber = accountPayee.SerialNumber;

                    accountDetailBalance.PayeeKvid = payment.PayerKvid;
                    accountDetailBalance.PayeeName = payment.PayerName;
                    accountDetailBalance.PayeeAccountKvid = accountPayer.Kvid;
                    accountDetailBalance.PayeeAccountName = accountPayer.Name;
                    accountDetailBalance.PayeeAccountSerialNumber = accountPayer.SerialNumber;

                    accountDetailBalance.AmountPayment = payment.Amount;
                    accountDetailBalance.Summary = "冲账:" + Remark;
                    #endregion
                    accountDetailBalance.Insert(conn);
                }

                trans.Commit();
            }
            return rtns;
        }
    }

    [Api(Description = "到账冲账撤销")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class PaymentOffsetCancel : RestfulExecution<Payment>
    {
        public Guid Kvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty).ThrowIfTrue("请传入要撤销冲账的到账Kvid!");
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payment = conn.SingleById<Payment>(Kvid);
            if (payment == null) throw new Exception("收付款项不存在");
            if (payment.Type != PaymentType.Bank && payment.Type != PaymentType.Cash && payment.Type != PaymentType.Pos && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat) throw new Exception("不支持的操作类型!");
            if (payment.OffsetKvid == Guid.Empty) throw new Exception("收付款未冲账,无需撤销");
            if (payment.BizKvid != Guid.Empty) throw new Exception("收付款已关联业务！请先解除关联！");
            payment.PayeeAccountKvid.ThrowIfEmpty("收款账户(PayeeAccountKvid)为空!");
            payment.PayerAccountKvid.ThrowIfEmpty("付款账户(PayerAccountKvid)为空!");
            var offset = conn.SingleById<Payment>(payment.OffsetKvid);

            var accountPayee = conn.SingleById<Account>(payment.PayeeAccountKvid);
            var accountPayer = conn.SingleById<Account>(payment.PayerAccountKvid);

            if (accountPayee == null) accountPayee = payment.Currency.GetBalanceAccount();
            if (accountPayer == null) accountPayer = payment.Currency.GetBalanceAccount();

            //payment 取消OffsetKvid
            payment.OffsetKvid = Guid.Empty;
            payment.AddOnlyProperties(o => o.OffsetKvid);

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            using (var trans = conn.OpenTransaction())
            {
                conn.UpdateOnly(payment);
                rtns.Results.Add(payment);
                if (offset != null)
                {
                    offset.Status = -1;
                    offset.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(offset);
                }

                if (accountPayee != null)
                {
                    #region 新增流水
                    var newAccountDetail = new AccountDetail();
                    newAccountDetail.AccountKvid = accountPayee.Kvid;
                    newAccountDetail.BizId = payment.SerialNumber;
                    newAccountDetail.BizKvid = payment.Kvid;
                    newAccountDetail.BizType = typeof(Payment).FullName;

                    newAccountDetail.PayeeKvid = payment.PayeeKvid;
                    newAccountDetail.PayeeName = payment.PayeeName;
                    newAccountDetail.PayeeAccountKvid = accountPayee.Kvid;
                    newAccountDetail.PayeeAccountName = accountPayee.Name;
                    newAccountDetail.PayeeAccountSerialNumber = accountPayee.SerialNumber;

                    newAccountDetail.PayerKvid = payment.PayerKvid;
                    newAccountDetail.PayerName = payment.PayerName;
                    newAccountDetail.PayerAccountKvid = accountPayer.Kvid;
                    newAccountDetail.PayerAccountName = accountPayer.Name;
                    newAccountDetail.PayerAccountSerialNumber = accountPayer.SerialNumber;

                    newAccountDetail.AmountPayment = payment.Amount;
                    newAccountDetail.Summary = "撤销冲账";
                    #endregion
                    newAccountDetail.Insert(conn);
                }
                if (accountPayer != null)
                {
                    #region 新增流水
                    var accountDetailBalance = new AccountDetail();
                    accountDetailBalance.AccountKvid = accountPayer.Kvid;
                    accountDetailBalance.BizId = payment.SerialNumber;
                    accountDetailBalance.BizKvid = payment.Kvid;
                    accountDetailBalance.BizType = typeof(Payment).FullName;

                    accountDetailBalance.PayeeKvid = payment.PayeeKvid;
                    accountDetailBalance.PayeeName = payment.PayeeName;
                    accountDetailBalance.PayeeAccountKvid = accountPayee.Kvid;
                    accountDetailBalance.PayeeAccountName = accountPayee.Name;
                    accountDetailBalance.PayeeAccountSerialNumber = accountPayee.SerialNumber;

                    accountDetailBalance.PayerKvid = payment.PayerKvid;
                    accountDetailBalance.PayerName = payment.PayerName;
                    accountDetailBalance.PayerAccountKvid = accountPayer.Kvid;
                    accountDetailBalance.PayerAccountName = accountPayer.Name;
                    accountDetailBalance.PayerAccountSerialNumber = accountPayer.SerialNumber;

                    accountDetailBalance.AmountPayment = payment.Amount;
                    accountDetailBalance.Summary = "撤销冲账";
                    #endregion
                    accountDetailBalance.Insert(conn);
                }

                trans.Commit();
            }
            return rtns;
        }
    }
}
